<?php
/**
* Prestashop Addons | Module by: <App1Pro>
*
* @author    Chuyen Nguyen [App1Pro].
* @copyright Chuyenim@gmail.com
* @license   http://app1pro.com/license.txt
*/

class ProductCommentProModel extends ObjectModel
{
    public $id_product_comment_pro_comment;
    public $id_parent = 0;
    public $id_product;
    public $id_customer;
    public $id_mention = 0;
    public $comment;
    public $email;
    public $name;
    public $status = 0;
    public $approved = 0;
    public $date_add;
    public $date_upd;
    const MAX_REPORTING = 3;

    public static $definition = array(
        'table' => 'product_comment_pro_comment',
        'primary' => 'id_product_comment_pro_comment',
        'multilang' => false,
        'fields' => array(
                'id_parent'         => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
                'id_product'        => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'required' => true),
                'id_customer'       => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
                'email'             => array('type' => self::TYPE_STRING, 'validate' => 'isEmail', 'size' => 200, 'required' => false),
                'name'              => array('type' => self::TYPE_STRING, 'validate' => 'isGenericName', 'size' => 100, 'required' => false),
                'id_mention'        => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
                'comment'           => array(
                    'type' => self::TYPE_HTML,
                    'validate' => 'isCleanHtml',
                    'required' => true,
                    'size' => 3999999999999
                ),
                'status'            => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
                'approved'          => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
                'date_add'          => array('type' => self::TYPE_DATE, 'validate' => 'isDateFormat'),
                'date_upd'          => array('type' => self::TYPE_DATE, 'validate' => 'isDateFormat'),
        ),
    );

    /**
     * Validate/Approve a comment
     *
     * @return boolean succeed
     */
    public function validate($validate = 1)
    {
        if (!Validate::isUnsignedId($this->id)) {
            return false;
        }
        $this->approved = (int)$validate;
        $this->status = 1;

        return $this->update();
    }

    public function delete()
    {
        if (parent::delete()) {
            $report = new ProductCommentProReport();
            return $report->deleteReports($this->id);
        }
        return false;
    }

    /**
     * Set Validate/Approve a comment
     *
     * @return self
     */
    public function validated($validate = 1)
    {
        $this->approved = (int)$validate;
        $this->status = (int)$validate;
        return $this;
    }

    public function deleteProd($id_product = 0)
    {
        $res = true;
        $sql = 'SELECT id_product_comment_pro_comment as id FROM `'._DB_PREFIX_.'product_comment_pro_comment`
                WHERE `id_product` = '.(int)$id_product.' OR `id_parent` ='.(int)$id_product;
        $comments = Db::getInstance()->executeS($sql);
        foreach ($comments as $row) {
            $comment = new self($row['id']);
            $res &= $comment->delete();
        }
        return $res;
    }

    public function countComments($id_product = 0, $id_parent = 0)
    {
        $sql = 'SELECT COUNT(*)
                FROM `'._DB_PREFIX_.'product_comment_pro_comment` a
                WHERE a.`status` = 1 AND a.`approved` = 1
                AND a.`id_product` = '.(int)$id_product;
        if ($id_parent !== false) {
            $sql .= ' AND a.`id_parent` = '.(int)$id_parent;
        }
        $sql .= ' ORDER BY a.`id_product_comment_pro_comment` ASC';

        return Db::getInstance()->getValue($sql);
    }

    public function getCustomer()
    {
        $sql = 'SELECT
                a.id_product_comment_pro_comment as id,
                COALESCE(pcpc.nickname, CONCAT(c.firstname, " ", c.lastname)) as nickname,
                c.email as email,
                pcpc.avatar
            FROM `'._DB_PREFIX_.'product_comment_pro_comment` a
            LEFT JOIN '._DB_PREFIX_.'product_comment_pro_customer pcpc ON (a.id_customer = pcpc.id_customer)
            LEFT JOIN '._DB_PREFIX_.'customer c ON (a.id_customer = c.id_customer)
            WHERE a.`id_product_comment_pro_comment` = '.(int)$this->id.' AND a.id_customer IS NOT NULL AND a.id_customer <> 0;';

        $result = Db::getInstance()->getRow($sql);
        return $result;
    }
    
    public function getComments($id_product = 0, $id_parent = 0, $id_customer = 0, array $pagination = array())
    {
        $sql = 'SELECT 
                a.id_product_comment_pro_comment as id,
                COALESCE(pcpc.nickname, CONCAT(c.firstname, " ", c.lastname), a.name)  as nickname,
                COALESCE(pcpcm.nickname, CONCAT(cm.firstname, " ", cm.lastname)) as mention,
                (
                    SELECT
                        COALESCE(COUNT(IF(`point` = 1, `point`, NULL)), 0)
                    FROM `'._DB_PREFIX_.'product_comment_pro_vote`
                    WHERE `id_comment` = a.id_product_comment_pro_comment
                ) as votes_up,
                (
                    SELECT
                        COALESCE(COUNT(IF(`point` = -1, `point`, NULL)), 0)
                    FROM `'._DB_PREFIX_.'product_comment_pro_vote`
                    WHERE `id_comment` = a.id_product_comment_pro_comment
                ) as votes_down,
                v.point as my_point,
                pcpc.avatar,
                a.*
            FROM `'._DB_PREFIX_.'product_comment_pro_comment` a
            LEFT JOIN '._DB_PREFIX_.'product_comment_pro_customer pcpc ON (a.id_customer = pcpc.id_customer)
            LEFT JOIN '._DB_PREFIX_.'customer c ON (a.id_customer = c.id_customer)
            LEFT JOIN '._DB_PREFIX_.'product_comment_pro_customer pcpcm ON (a.id_mention = pcpcm.id_customer)
            LEFT JOIN '._DB_PREFIX_.'customer cm ON (a.id_mention = cm.id_customer)
            LEFT JOIN '._DB_PREFIX_.'product_comment_pro_vote v
            ON (
                a.id_product_comment_pro_comment = v.id_comment
                AND v.id_customer = '.(int)$id_customer.'
            )
            WHERE a.`status` = 1 AND a.`approved` = 1
            AND a.`id_product` = '.(int)$id_product.'
            AND a.`id_parent` = '.(int)$id_parent;
        $sql .= ' GROUP BY a.`id_product_comment_pro_comment`
            ORDER BY a.`id_product_comment_pro_comment` ASC';
        if (!empty($pagination)) {
            $sql .= ' LIMIT '.(int)$pagination['limit'].' OFFSET '.(int)$pagination['offset'];
        }

        $result = Db::getInstance()->executeS($sql);
        if (!empty($result)) {
            foreach ($result as $key => $row) {
                if ($this->hasChildren($row['id'])) {
                    $result[$key]['children'] = $this->getComments($id_product, $row['id'], $id_customer);
                }
            }
        }
        return $result;
    }
    
    public function hasChildren($id_comment = 0)
    {
        $sql = 'SELECT a.* FROM `'._DB_PREFIX_.'product_comment_pro_comment` a
                WHERE a.`status` = 1 AND a.`approved` = 1 AND a.`id_parent` = '.(int)$id_comment;
        
        $result = Db::getInstance()->executeS($sql);
        return !empty($result);
    }

    public static function getAll($id_product = 0, $approved = false, $reported = false)
    {
        $id_shop = Context::getContext()->shop->id;
        $id_lang = Context::getContext()->language->id;
        
        $sql = 'SELECT 
                a.id_product_comment_pro_comment as id,
                CONCAT(c.firstname, " ", c.lastname) as nickname,
                pl.name as product_name,
                cpr.reported,
                a.*
            FROM `'._DB_PREFIX_.'product_comment_pro_comment` a
            LEFT JOIN '._DB_PREFIX_.'customer c ON (a.id_customer = c.id_customer)
            LEFT JOIN (
                SELECT COUNT(*) as reported, id_comment
                FROM '._DB_PREFIX_.'product_comment_pro_report 
                GROUP BY `id_comment`
            ) as cpr ON (a.`id_product_comment_pro_comment` = cpr.id_comment)
            LEFT JOIN '._DB_PREFIX_.'product_lang pl ON (a.id_product = pl.id_product AND pl.id_lang = '.(int)$id_lang.' AND pl.id_shop = '.(int)$id_shop.')
            WHERE a.`approved` = '.(int)$approved;
        if ($id_product) {
            $sql .= ' AND a.`id_product` = '.(int)$id_product;
        }
        if ($reported) {
            $sql .= ' AND `reported` >= '.(int)self::MAX_REPORTING;
            $sql .= ' ORDER BY a.`status` DESC, `reported` DESC;';
        } else {
            $sql .= ' ORDER BY a.`id_product_comment_pro_comment` DESC;';
        }
        

        return Db::getInstance()->executeS($sql);
    }
}
