<?php
/**
* Prestashop Addons | Module by: <App1Pro>
*
* @author    Chuyen Nguyen [App1Pro].
* @copyright Chuyenim@gmail.com
* @license   http://app1pro.com/license.txt
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

define('_MODULE_PCP_URI_', __PS_BASE_URI__.'modules/productcommentpro/');
define('_MODULE_PCP_AVT_DIR_', _PS_MODULE_DIR_.'productcommentpro/views/img/avatars/');
define('_MODULE_PCP_AVT_URI_', __PS_BASE_URI__.'modules/productcommentpro/views/img/avatars/');

require_once(dirname(__FILE__).'/classes/ProductCommentProModel.php');
require_once(dirname(__FILE__).'/classes/ProductCommentProCustomer.php');
require_once(dirname(__FILE__).'/classes/ProductCommentProNotif.php');
require_once(dirname(__FILE__).'/classes/ProductCommentProReport.php');
require_once(dirname(__FILE__).'/classes/ProductCommentProVote.php');

class ProductCommentPro extends Module
{
    public function __construct()
    {
        $this->name = 'productcommentpro';
        $this->tab = 'content_management';
        $this->version = '1.4.0';
        $this->author = 'App1pro';
        $this->need_instance = 0;
        $this->need_upgrade = true;

        $this->bootstrap = true;

        parent::__construct();
        $this->displayName = $this->l('Products Comments Pro');
        $this->description = $this->l('This module allows customers send messages in product pages.');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall this module?');
        $this->temp_layout = '/views/templates/hooks/product_footer.tpl';
    }

    public function install()
    {
        Configuration::updateValue('PRODUCTCOMMENTPRO_ADMIN_GROUP', 1);
        Configuration::updateValue('PRODUCTCOMMENTPRO_MUST_APPR', 1);
        Configuration::updateValue('PRODUCTCOMMENTPRO_ROP', 20);
        Configuration::updateValue('PRODUCTCOMMENTPRO_FB_ENABLED', 0);
        Configuration::updateValue('PRODUCTCOMMENTPRO_FB_APPID', '843470559035505');
        $languages = Language::getLanguages(false);

        foreach ($languages as $lang) {
            $this->installFixture((int)$lang['id_lang'], 'Comments');
        }

        if (!parent::install()
        || !$this->registerHook('displayHeader')
        || !$this->registerHook('actionProductDelete')
        || !$this->registerHook('actionObjectLanguageAddAfter')
        || !$this->registerHook('displayCustomerAccount')
        || !$this->registerHook('actionCustomerAccountAdd')
        || !$this->registerHook('displayFooterProduct')) {
            return false;
        }

        require_once(dirname(__FILE__).'/sql/install.php');
        return true;
    }

    protected function installFixture($id_lang, $title = null)
    {
        $values = array();
        $values['title'][(int)$id_lang] = $title;
        Configuration::updateValue('PRODUCTCOMMENTPRO_TITLE', $values['title']);
    }

    public function hookActionObjectLanguageAddAfter($params)
    {
        $title = Configuration::get('PRODUCTCOMMENTPRO_TITLE', (int)Configuration::get('PS_LANG_DEFAULT'));
        return $this->installFixture((int)$params['object']->id, $title);
    }

    public function uninstall()
    {
        Configuration::deleteByName('PRODUCTCOMMENTPRO_TITLE');
        Configuration::deleteByName('PRODUCTCOMMENTPRO_ADMIN_GROUP');
        Configuration::deleteByName('PRODUCTCOMMENTPRO_MUST_APPR');
        Configuration::deleteByName('PRODUCTCOMMENTPRO_ROP');
        Configuration::deleteByName('PRODUCTCOMMENTPRO_FB_ENABLED');
        Configuration::deleteByName('PRODUCTCOMMENTPRO_FB_APPID');

        require_once(dirname(__FILE__).'/sql/uninstall.php');
        return parent::uninstall();
    }

    public function hookDisplayHeader()
    {
        $this->context->controller->addJquery();
        $this->context->controller->addjqueryPlugin('fancybox');
        $lang_iso_code = $this->context->language->iso_code;
        $this->context->controller->addCSS(($this->_path).'views/css/style.css', 'all');
        if (Tools::getValue('controller') === 'product') {
            $this->context->controller->addJS(($this->_path).'libraries/jquery.livequery.min.js');
            $this->context->controller->addJS(($this->_path).'libraries/jquery.timeago.min.js');
            $this->context->controller->addJS(($this->_path).'libraries/locales/jquery.timeago.'.$lang_iso_code.'.js');
            
            // if (version_compare(_PS_VERSION_, '1.7.0.0 ', '>=')) {
            //     $this->context->controller->registerJavascript(null, '//cdn.tinymce.com/4/tinymce.min.js', ['server' => 'remote']);
            // } else {
            //     $this->context->controller->addJS('//cdn.tinymce.com/4/tinymce.min.js');
            // }
            $this->context->controller->addJS(($this->_path).'views/js/product_footer.min.js');
        } elseif (Tools::getValue('controller') === 'myaccount') {
            $this->context->controller->addJS(($this->_path).'views/js/customer_identity.min.js');
        }
    }

    public function getContent()
    {
        if (Tools::isSubmit('updateproduct_comment_pro_comment')) {
            return $this->postProcess().$this->renderCommentForm().$this->renderReportList();
        }
        return $this->postProcess()
            .$this->renderForm()
            .$this->renderWaitingList()
            .$this->renderBadCommentList()
            .$this->renderApprovedList();
    }

    public function hookActionProductDelete($params)
    {
        $id_product = (int)$params['product']->id;
        $comment = new ProductCommentProModel();
        return $comment->deleteProd($id_product);
    }

    public function hookActionCustomerAccountAdd($params)
    {
        $customer = $params['newCustomer'];
        $pcpaccount = new ProductCommentProCustomer();

        if (!empty($customer)) {
            $pcpaccount->id_customer = $customer->id;
            $pcpaccount->nickname = $customer->firstname.' '.$customer->lastname;
            return $pcpaccount->save();
        }
    }

    public function hookDisplayCustomerAccount()
    {
        $id_lang = $this->context->language->id;
        $customer = $this->context->customer;
        $id_customer = $this->context->customer->id;
        $account = new ProductCommentProCustomer($id_customer);
        $notif = new ProductCommentProNotif();
        $notifs = $notif->getNotifs($id_customer, $id_lang);
        $count_notifs = $notif->countUnreadNotifs($id_customer);
        
        if ($account && !empty($account->avatar)) {
            $account->avatar_url = _MODULE_PCP_AVT_URI_.$account->avatar;
        } else {
            $account->avatar_url = _MODULE_PCP_AVT_URI_.'default.png';
        }
        
        $notifs = array_map(function (array $notif) {
            $notif['url'] = $this->context->link->getProductLink($notif['id_product']);
            return $notif;
        }, $notifs);
        
        $this->context->smarty->assign(array(
            'pcp_url'  => $this->context->link->getModuleLink($this->name, 'ajax'),
            'nickname' => $customer->firstname.' '.$customer->lastname,
            '_path'    => $this->_path,
            'notifs'   => $notifs,
            'count_notifs' => $count_notifs,
            'account'  => (array)$account,
            'version_17' => version_compare(_PS_VERSION_, '1.7.0.0 ', '>=')
        ));

        return $this->display(__file__, '/views/templates/hooks/customer_identity.tpl');
    }

    public function hookDisplayProductTab()
    {
        return $this->display(__file__, '/views/templates/hooks/product_tab.tpl');
    }

    public function hookDisplayProductTabContent()
    {
        $this->temp_layout = '/views/templates/hooks/product_tab_content.tpl';
        return $this->hookDisplayFooterProduct();
    }

    public function hookDisplayFooterProduct()
    {
        $pcp_title = Configuration::get('PRODUCTCOMMENTPRO_TITLE', $this->context->language->id);
        $pcp_ad_group = Configuration::get('PRODUCTCOMMENTPRO_ADMIN_GROUP');
        $id_product = Tools::getValue('id_product');
        $id_customer = $this->context->customer->id;
        $id_default_group = $this->context->customer->id_default_group;
        $is_pcpadmin = $id_customer && $pcp_ad_group && ($id_default_group == $pcp_ad_group);
        
        $model = new ProductCommentProModel();
        
        // Count all comments and its replies.
        $no_comments = $model->countComments($id_product, false);

        // Update notif for this customer
        if ($id_customer) {
            $notif = new ProductCommentProNotif();
            $notif->updateStatus($id_customer);
        }
        
        // Count all comments at level 0.
        $no0_comments = $model->countComments($id_product, 0);
        $current_page = 1;
        $record_per_page = (int)Configuration::get('PRODUCTCOMMENTPRO_ROP');
        $max_page = $record_per_page ? ceil($no0_comments / $record_per_page) : 0;
        $pagination = array(
            'offset' => $record_per_page * ($current_page - 1),
            'limit' => $record_per_page,
        );
        $comments = $model->getComments($id_product, 0, $id_customer, $pagination);
        // pd($comments);
        
        $this->context->smarty->assign(array(
            'pcp_title'     => $pcp_title,
            'is_pcpadmin'   => $is_pcpadmin,
            'id_customer'   => $id_customer,
            'id_product'    => $id_product,
            'fb_comment_on' => Configuration::get('PRODUCTCOMMENTPRO_FB_ENABLED'),
            'fb_app_id'     => Configuration::get('PRODUCTCOMMENTPRO_FB_APPID'),
            'pcp_url'       => $this->context->link->getModuleLink($this->name, 'ajax'),
            'login_url'     => $this->context->link->getPageLink('authentication', true),
            'register_url'  => $this->context->link->getPageLink('authentication', true, null, array('create_account' => '1')),
            'comments'      => $comments,
            'no_comments'   => $no_comments,
            'max_page'      => $max_page,
            'current_page'  => $current_page,
            '_path'         => $this->_path
        ));

        return $this->display(__file__, $this->temp_layout);
    }

    public function postProcess()
    {
        if (Tools::isSubmit('submitPCPConf')) {
            $values = array();
            $languages = Language::getLanguages(false);

            foreach ($languages as $lang) {
                $values['PRODUCTCOMMENTPRO_TITLE'][$lang['id_lang']] = Tools::getValue('PRODUCTCOMMENTPRO_TITLE_'.$lang['id_lang']);
            }
            
            if (!Validate::isUnsignedId(Tools::getValue('PRODUCTCOMMENTPRO_ROP')) || Tools::getValue('PRODUCTCOMMENTPRO_ROP') == 0) {
                $this->errors[] = Tools::displayError('The "Number of Comment per page" field is invalid. (Must be an integer greater than 0)');
            }

            if (!empty($this->errors)) {
                return $this->displayError(implode('\n', $this->errors));
            } else {
                Configuration::updateValue('PRODUCTCOMMENTPRO_TITLE', $values['PRODUCTCOMMENTPRO_TITLE']);
                Configuration::updateValue('PRODUCTCOMMENTPRO_ADMIN_GROUP', (int)Tools::getValue('PRODUCTCOMMENTPRO_ADMIN_GROUP'));
                Configuration::updateValue('PRODUCTCOMMENTPRO_MUST_APPR', (bool)Tools::getValue('PRODUCTCOMMENTPRO_MUST_APPR'));
                Configuration::updateValue('PRODUCTCOMMENTPRO_ROP', (int)Tools::getValue('PRODUCTCOMMENTPRO_ROP'));
                Configuration::updateValue('PRODUCTCOMMENTPRO_FB_ENABLED', (bool)Tools::getValue('PRODUCTCOMMENTPRO_FB_ENABLED'));
                Configuration::updateValue('PRODUCTCOMMENTPRO_FB_APPID', Tools::getValue('PRODUCTCOMMENTPRO_FB_APPID'));
            }

            $this->_clearCache('/views/templates/hooks/productfooter.tpl');
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules').'&configure='.$this->name.'&conf=4');
        } elseif ($id_comment = (int)Tools::getValue('approveComment')) {
            $comment = new ProductCommentProModel($id_comment);
            if ($comment->validate()) {
                // If is reply to another message then send notif to author
                if ($comment->id_parent) {
                    $this->prepareNotif(ProductCommentProNotif::TYPE_REPLY, $comment->id_parent, nl2br($comment->comment), $comment->id_customer);
                }
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules').'&configure='.$this->name);
            }
        } elseif (Tools::isSubmit('deleteproduct_comment_pro_comment')) {
            $id_comment = (int)Tools::getValue('id_product_comment_pro_comment');
            $comment = new ProductCommentProModel($id_comment);
            if ($comment->delete()) {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules').'&configure='.$this->name.'&conf=2');
            }
        } elseif (Tools::isSubmit('deleteproduct_comment_pro_report')) {
            $id_report = (int)Tools::getValue('id_product_comment_pro_report');
            $report = new ProductCommentProReport($id_report);
            if ($report->delete()) {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules').'&configure='.$this->name.'&conf=2');
            }
        } elseif (Tools::isSubmit('statusproduct_comment_pro_comment')) {
            $id_comment = (int)Tools::getValue('id_product_comment_pro_comment');
            $comment = new ProductCommentProModel($id_comment);
            $comment->status = !$comment->status;
            if ($comment->update()) {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules').'&configure='.$this->name.'&conf=4');
            }
        } elseif (Tools::isSubmit('submitBulkdeleteproduct_comment_pro_comment')) {
            $table = 'product_comment_pro_comment';
            foreach (Tools::getValue($table.'Box') as $selection) {
                $comment = new ProductCommentProModel((int)$selection);
                if (!$comment->delete()) {
                    $this->errors[] = $this->l('Runtime error.');
                    break;
                }
            }
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&configure='.$this->name.'&conf=2');
        } elseif (Tools::isSubmit('submitBulkapproveproduct_comment_pro_comment')) {
            $table = 'product_comment_pro_comment';
            foreach (Tools::getValue($table.'Box') as $selection) {
                $comment = new ProductCommentProModel((int)$selection);
                if (!$comment->validate()) {
                    $this->errors[] = $this->l('Runtime error.');
                    break;
                }
            }
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&configure='.$this->name);
        } elseif (Tools::isSubmit('submitCommentEdit')) {
            $id_comment = (int)Tools::getValue('id_product_comment_pro_comment');
            $comment = new ProductCommentProModel($id_comment);
            $comment->comment = Tools::getValue('comment');
            $comment->id_customer = Tools::getValue('id_customer');
            $comment->email = Tools::getValue('email');
            $comment->name = Tools::getValue('name');
            $comment->status = Tools::getValue('status');
            $comment->approved = Tools::getValue('approved');
            $comment->date_add = Tools::getValue('date_add');
            // Check validate data
            $this->checkValidation($_POST, 'ProductCommentProModel');
            if (!empty($this->errors)) {
                return $this->displayError(implode('\n', $this->errors));
            }
            
            if ($comment->update()) {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true).'&configure='.$this->name.'&conf=4');
            }
        }
        return '';
    }

    /*
    * Prepare a notification email
    */
    public function prepareNotif($type, $id_comment = 0, $response = null, $id_ignore = 0)
    {
        if (!$id_comment) {
            return false;
        }
        $pcp_comment = new ProductCommentProModel($id_comment);
        $id_author = $pcp_comment->id_customer;
        if ($id_author == $id_ignore) {
            return false;
        }
        $pcp_author = new ProductCommentProCustomer($id_author);

        $notif = new ProductCommentProNotif();
        $notif->id_product = $pcp_comment->id_product;
        $notif->id_comment = $id_comment;
        $notif->id_customer = $id_ignore;
        $notif->type = $type;
        $notif->date_add = date('Y-m-d H:i:s');
        $notif->add();
        
        if ($pcp_author->mailing) {
            $customer = new Customer($id_author);

            $id_product = $pcp_comment->id_product;
            $product = new Product($id_product, false, (int)$customer->id_lang);
            $link = $this->context->link->getProductLink($product);

            $var_list = array(
                '{nickname}' => (!empty($pcp_author->nickname)) ? $pcp_author->nickname : $customer->firstname.' '.$customer->lastname,
                '{comment}' => nl2br($pcp_comment->comment),
                '{response}' => $response,
                '{link}' => $link
            );
            ProductCommentProNotif::sendNotifMail($customer->email, $customer->id_lang, 'new_response', Mail::l('Someone reacted to your comment', $customer->id_lang), $var_list);
        }
    }

    public function checkValidation(array $data, $className)
    {
        $rules = call_user_func(array($className, 'getValidationRules'), $className);

        // Check required fields
        foreach ($rules['required'] as $field) {
            if (isset($data[$field]) && ($value = $data[$field]) == false && $value != '0') {
                $this->errors[] = sprintf(
                    Tools::displayError('The %s field is required.'),
                    call_user_func(array($className, 'displayFieldName'), $field, $className)
                );
            }
        }

        // Check fields validity
        foreach ($rules['validate'] as $field => $function) {
            if (isset($data[$field]) && $value = $data[$field]) {
                $res = true;
                if (!Validate::{$function}($value)) {
                    $res = false;
                }

                if (!$res) {
                    $this->errors[] = sprintf(
                        Tools::displayError('The %s field is invalid.'),
                        call_user_func(array($className, 'displayFieldName'), $field, $className)
                    );
                }
            }
        }
    }

    public function renderForm()
    {
        $type_radio = 'radio';
        if (version_compare(_PS_VERSION_, '1.6.0.0 ', '>=')) {
            $type_radio = 'switch';
        }
        $groups = Group::getGroups($this->context->language->id, true);
        
        $visitor_group = Configuration::get('PS_UNIDENTIFIED_GROUP');
        $guest_group = Configuration::get('PS_GUEST_GROUP');
        foreach ($groups as $key => $g) {
            if (in_array($g['id_group'], array($visitor_group, $guest_group))) {
                unset($groups[$key]);
            }
        }
        
        array_unshift($groups, array('id_group' => null, 'name' => '-- Select one --'));
        
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Common Settings'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => $this->l('Title'),
                        'name' => 'PRODUCTCOMMENTPRO_TITLE',
                        'required' => true,
                        'lang' => true,
                        'size' => 50
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Moderators Group'),
                        'name' => 'PRODUCTCOMMENTPRO_ADMIN_GROUP',
                        'options' => array(
                            'query' => $groups,
                            'id' => 'id_group',
                            'name' => 'name'
                        ),
                        'col' => '4',
                        'desc' => '<a href="'.$this->context->link->getAdminLink('AdminGroups').'&amp;addgroup'.'" target=_blank>'.
                            $this->l('Create a new group').' <i class="icon-external-link"></i></a>'
                    ),
                    array(
                        'type' => $type_radio,
                        'label' => $this->l('All reviews must be validated by an admin'),
                        'name' => 'PRODUCTCOMMENTPRO_MUST_APPR',
                        'class' => 't',
                        'values' => array(
                                array(
                                    'id' => 'must_appr_on',
                                    'value' => 1,
                                    'label' => $this->l('Enabled')
                                ),
                                array(
                                    'id' => 'must_appr_off',
                                    'value' => 0,
                                    'label' => $this->l('Disabled')
                                )
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Number of comments per page'),
                        'name' => 'PRODUCTCOMMENTPRO_ROP',
                        'required' => true,
                        'size' => 50
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            ),
        );

        $fb_setting_html = $this->context->smarty->fetch(dirname(__FILE__).'/views/templates/hooks/fb_setting_html.tpl');
        $fields_fbform = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Facebook Comment Settings'),
                    'icon' => 'icon-cogs'
                ),
                'description' => $fb_setting_html,
                'input' => array(
                    array(
                        'type' => $type_radio,
                        'label' => $this->l('Facebook Comment'),
                        'name' => 'PRODUCTCOMMENTPRO_FB_ENABLED',
                        'class' => 't',
                        'values' => array(
                                array(
                                    'id' => 'fb_on',
                                    'value' => 1,
                                    'label' => $this->l('Enabled')
                                ),
                                array(
                                    'id' => 'fb_off',
                                    'value' => 0,
                                    'label' => $this->l('Disabled')
                                )
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Facebook App ID'),
                        'name' => 'PRODUCTCOMMENTPRO_FB_APPID',
                        'required' => true,
                        'size' => 50
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            ),
        );

        if (version_compare(_PS_VERSION_, '1.6.0.0 ', '<')) {
            $fields_form['form']['submit']['class'] = 'button';
            $fields_fbform['form']['submit']['class'] = 'button';
        }

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;

        $employee_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG');
        $helper->allow_employee_form_lang = $employee_lang ? $employee_lang : 0;
        //$this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitPCPConf';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
                .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form, $fields_fbform));
    }

    public function getConfigFieldsValues()
    {
        $fields = array();
        $languages = Language::getLanguages(false);

        foreach ($languages as $lang) {
            $fields['PRODUCTCOMMENTPRO_TITLE'][$lang['id_lang']] = Tools::getValue(
                'PRODUCTCOMMENTPRO_TITLE_'.$lang['id_lang'],
                Configuration::get('PRODUCTCOMMENTPRO_TITLE', $lang['id_lang'])
            );
        }

        $fields['PRODUCTCOMMENTPRO_ADMIN_GROUP'] = Tools::getValue(
            'PRODUCTCOMMENTPRO_ADMIN_GROUP',
            Configuration::get('PRODUCTCOMMENTPRO_ADMIN_GROUP')
        );

        $fields['PRODUCTCOMMENTPRO_MUST_APPR'] = Tools::getValue(
            'PRODUCTCOMMENTPRO_MUST_APPR',
            Configuration::get('PRODUCTCOMMENTPRO_MUST_APPR')
        );

        $fields['PRODUCTCOMMENTPRO_ROP'] = Tools::getValue(
            'PRODUCTCOMMENTPRO_ROP',
            Configuration::get('PRODUCTCOMMENTPRO_ROP')
        );

        $fields['PRODUCTCOMMENTPRO_FB_ENABLED'] = Tools::getValue(
            'PRODUCTCOMMENTPRO_FB_ENABLED',
            Configuration::get('PRODUCTCOMMENTPRO_FB_ENABLED')
        );

        $fields['PRODUCTCOMMENTPRO_FB_APPID'] = Tools::getValue(
            'PRODUCTCOMMENTPRO_FB_APPID',
            Configuration::get('PRODUCTCOMMENTPRO_FB_APPID')
        );

        return $fields;
    }
    
    public function renderWaitingList()
    {
        $fields_list = array(
            'id_product_comment_pro_comment' => array(
                'title' => $this->l('ID'),
                'search' => false,
            ),
            'nickname' => array(
                'title' => $this->l('Author'),
                'search' => false,
            ),
            'product_name' => array(
                'title' => $this->l('Product Name'),
                'maxlength' => 50,
                'search' => false,
            ),
            'comment' => array(
                'title' => $this->l('Comment'),
                'type' => 'text',
                'maxlength' => 70,
                'search' => false,
            ),
            'date_add' => array(
                'title' => $this->l('Date created'),
                'type' => 'datetime',
                'search' => false,
            )
        );

        $helper_list = new HelperList();
        $helper_list->module = $this;
        $helper_list->title = $this->l('Comments waiting for approval');
        $helper_list->shopLinkType = '';
        $helper_list->no_link = true;
        $helper_list->show_toolbar = false;
        // $helper_list->simple_header = true;
        $helper_list->identifier = 'id_product_comment_pro_comment';
        $helper_list->table = 'product_comment_pro_comment';
        $helper_list->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name;
        $helper_list->token = Tools::getAdminTokenLite('AdminModules');
        $helper_list->actions = array('approve', 'edit', 'delete');
        $helper_list->bulk_actions = array(
            'approve' => array(
                'text' => $this->l('Approve selected'),
                'confirm' => $this->l('Approve selected items?'),
                'icon' => 'icon-check'
            ),
            'delete' => array(
                'text' => $this->l('Delete selected'),
                'confirm' => $this->l('Delete selected items?'),
                'icon' => 'icon-trash'
            )
        );

        // This is needed for displayEnableLink to avoid code duplication
        $this->_helperlist = $helper_list;

        /* Retrieve list data */
        $comments = ProductCommentProModel::getAll(false, false);
        $helper_list->listTotal = count($comments);
        
//        echo '<pre>';
//        print_r($comments);
//        echo '</pre>';
//        die;
        
        /* Paginate the result */
        $page = ($page = Tools::getValue('submitFilter'.$helper_list->table)) ? $page : 1;
        $pagination = ($pagination = Tools::getValue($helper_list->table.'_pagination')) ? $pagination : 50;
        $comments = $this->paginate($comments, $page, $pagination);

        return $helper_list->generateList($comments, $fields_list);
    }

    public function paginate($list, $page = 1, $pagination = 50)
    {
        if (count($list) > $pagination) {
            $list = array_slice($list, $pagination * ($page - 1), $pagination);
        }

        return $list;
    }

    public function displayApproveLink($token = null, $id = 0, $name = null)
    {
        $this->smarty->assign(array(
            'href' => $this->context->link->getAdminLink('AdminModules').'&configure='.$this->name.'&module_name='.$this->name.'&approveComment='.$id,
            'action' => $this->l('Approve'),
        ));
        unset($token);
        unset($name);

        return $this->display(__FILE__, 'views/templates/admin/list_action_approve.tpl');
    }

    public function renderBadCommentList()
    {
        $fields_list = array(
            'id_product_comment_pro_comment' => array(
                'title' => $this->l('ID'),
                'search' => false,
            ),
            'nickname' => array(
                'title' => $this->l('Author'),
                'search' => false,
            ),
            'product_name' => array(
                'title' => $this->l('Product Name'),
                'maxlength' => 50,
                'search' => false,
            ),
            'comment' => array(
                'title' => $this->l('Comment'),
                'type' => 'text',
                'maxlength' => 70,
                'search' => false,
            ),
            'reported' => array(
                'title' => $this->l('Reported'),
                'search' => false,
            ),
            'status' => array(
                'title' => $this->l('Status'),
                'active' => 'status',
                'type' => 'bool',
                'search' => false,
            ),
            'date_add' => array(
                'title' => $this->l('Date created'),
                'type' => 'datetime',
                'search' => false,
            )
        );

        $helper_list = new HelperList();
        $helper_list->module = $this;
        $helper_list->title = sprintf($this->l('Comments were reported more than %s times'), ProductCommentProModel::MAX_REPORTING);
        $helper_list->shopLinkType = '';
        $helper_list->no_link = true;
        $helper_list->show_toolbar = true;
        // $helper_list->simple_header = true;
        $helper_list->identifier = 'id_product_comment_pro_comment';
        $helper_list->table = 'product_comment_pro_comment';
        $helper_list->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name;
        $helper_list->token = Tools::getAdminTokenLite('AdminModules');
        $helper_list->actions = array('edit', 'delete');
        $helper_list->bulk_actions = array(
            'delete' => array(
                'text' => $this->l('Delete selected'),
                'confirm' => $this->l('Delete selected items?'),
                'icon' => 'icon-trash'
            )
        );

        // This is needed for displayEnableLink to avoid code duplication
        $this->_helperlist = $helper_list;

        /* Retrieve list data */
        $comments = ProductCommentProModel::getAll(false, true, true);
        $helper_list->listTotal = count($comments);

        /* Paginate the result */
        $page = ($page = Tools::getValue('submitFilter'.$helper_list->table)) ? $page : 1;
        $pagination = ($pagination = Tools::getValue($helper_list->table.'_pagination')) ? $pagination : 50;
        $comments = $this->paginate($comments, $page, $pagination);

        return $helper_list->generateList($comments, $fields_list);
    }

    public function renderApprovedList()
    {
        $fields_list = array(
            'id_product_comment_pro_comment' => array(
                'title' => $this->l('ID'),
                'search' => true,
            ),
            'nickname' => array(
                'title' => $this->l('Author'),
                'search' => false,
            ),
            'product_name' => array(
                'title' => $this->l('Product Name'),
                'maxlength' => 50,
                'search' => true,
            ),
            'comment' => array(
                'title' => $this->l('Comment'),
                'type' => 'text',
                'maxlength' => 70,
                'search' => true,
            ),
            'reported' => array(
                'title' => $this->l('Reported'),
                'search' => false,
            ),
            'status' => array(
                'title' => $this->l('Status'),
                'active' => 'status',
                'type' => 'bool',
                'search' => false,
            ),
            'date_add' => array(
                'title' => $this->l('Date created'),
                'type' => 'datetime',
                'search' => false,
            )
        );

        $helper_list = new HelperList();
        $helper_list->module = $this;
        $helper_list->title = $this->l('Approved Comments');
        $helper_list->shopLinkType = '';
        $helper_list->no_link = true;
        $helper_list->show_toolbar = true;
        // $helper_list->simple_header = true;
        $helper_list->identifier = 'id_product_comment_pro_comment';
        $helper_list->table = 'product_comment_pro_comment';
        $helper_list->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name;
        $helper_list->token = Tools::getAdminTokenLite('AdminModules');
        $helper_list->actions = array('edit', 'delete');
        $helper_list->bulk_actions = array(
            'delete' => array(
                'text' => $this->l('Delete selected'),
                'confirm' => $this->l('Delete selected items?'),
                'icon' => 'icon-trash'
            )
        );

        // This is needed for displayEnableLink to avoid code duplication
        $this->_helperlist = $helper_list;

        /* Retrieve list data */
        $comments = ProductCommentProModel::getAll(false, true);
        $helper_list->listTotal = count($comments);

        /* Paginate the result */
        $page = ($page = Tools::getValue('submitFilter'.$helper_list->table)) ? $page : 1;
        $pagination = ($pagination = Tools::getValue($helper_list->table.'_pagination')) ? $pagination : 50;
        $comments = $this->paginate($comments, $page, $pagination);

        return $helper_list->generateList($comments, $fields_list);
    }

    public function renderCommentForm()
    {
        $model = new ProductCommentProModel((int)Tools::getValue('id_product_comment_pro_comment'));
        $comment = $model->getCustomer();
        
        $type_radio = 'radio';
        if (version_compare(_PS_VERSION_, '1.6.0.0 ', '>=')) {
            $type_radio = 'switch';
        }
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Update'),
                    'icon' => 'icon-edit'
                ),

                'input' => array(
                    array(
                        'type' => 'textarea',
                        'label' => $this->l('Comment'),
                        'name' => 'comment',
                        'required' => true,
                        'rows' => 5,
                        'size' => 150,
                    ),
                    
                    array(
                        'type' => 'text',
                        'label' => $this->l('Customer author ID'),
                        'name' => 'id_customer',
                        'desc' => !empty($comment) ? $comment['nickname'].' / '.$comment['email'] : null,
                        'size' => 200
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Email'),
                        'name' => 'email',
                        'size' => 200
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Name'),
                        'name' => 'name',
                        'size' => 100
                    ),
                    
                    array(
                        'type' => $type_radio,
                        'label' => $this->l('Status'),
                        'name' => 'status',
                        'class' => 't',
                        'values' => array(
                                array(
                                    'id' => 'status_on',
                                    'value' => 1,
                                    'label' => $this->l('Enabled')
                                ),
                                array(
                                    'id' => 'status_off',
                                    'value' => 0,
                                    'label' => $this->l('Disabled')
                                )
                        ),
                    ),
                    array(
                        'type' => $type_radio,
                        'label' => $this->l('Approved'),
                        'name' => 'approved',
                        'class' => 't',
                        'values' => array(
                                array(
                                    'id' => 'approved_on',
                                    'value' => 1,
                                    'label' => $this->l('Enabled')
                                ),
                                array(
                                    'id' => 'approved_off',
                                    'value' => 0,
                                    'label' => $this->l('Disabled')
                                )
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Created date'),
                        'name' => 'date_add',
                        'required' => true
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            ),
        );

        if (version_compare(_PS_VERSION_, '1.6.0.0 ', '<')) {
            $fields_form['form']['submit']['class'] = 'button';
        }

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->id = (int)Tools::getValue('id_product_comment_pro_comment');
        $helper->submit_action = 'submitCommentEdit';
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->show_cancel_button = true;
        $comment = new ProductCommentProModel($helper->id);
        $helper->tpl_vars = array(
            'fields_value' => (array)$comment,
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }

    public function renderReportList()
    {
        $fields_list = array(
            'id_product_comment_pro_report' => array(
                'title' => $this->l('ID'),
                'search' => false,
            ),
            'nickname' => array(
                'title' => $this->l('Author'),
                'search' => false,
            ),
            'reason' => array(
                'title' => $this->l('Reason'),
                'type' => 'text',
                'search' => false,
            ),
            'date_add' => array(
                'title' => $this->l('Date created'),
                'type' => 'datetime',
                'search' => false,
            )
        );

        $helper_list = new HelperList();
        $helper_list->module = $this;
        $helper_list->title = $this->l('Reports');
        $helper_list->shopLinkType = '';
        $helper_list->no_link = true;
        $helper_list->show_toolbar = true;
        // $helper_list->simple_header = true;
        $helper_list->identifier = 'id_product_comment_pro_report';
        $helper_list->table = 'product_comment_pro_report';
        $helper_list->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name;
        $helper_list->token = Tools::getAdminTokenLite('AdminModules');
        $helper_list->actions = array('delete');


        // This is needed for displayEnableLink to avoid code duplication
        $this->_helperlist = $helper_list;
        $id_comment = Tools::getValue('id_product_comment_pro_comment');

        /* Retrieve list data */
        $reports = ProductCommentProReport::getAll($id_comment);
        $helper_list->listTotal = count($reports);

        /* Paginate the result */
        $page = ($page = Tools::getValue('submitFilter'.$helper_list->table)) ? $page : 1;
        $pagination = ($pagination = Tools::getValue($helper_list->table.'_pagination')) ? $pagination : 50;
        $reports = $this->paginate($reports, $page, $pagination);

        return $helper_list->generateList($reports, $fields_list);
    }
}
